#include <string.h>
#include <stdlib.h>
#include <iostream.h>

#include "compareFiles.h"
#include "alignment.h"


void compareFiles::printStatisticsFileColumns(int numAminos, float *compareVect) {

  int i;

  cout << "Num.Col\tValue" << endl;
  for(i = 0; i < numAminos; i++) 
    cout << setw(5) << i+1 << "\t" << compareVect[i] << endl;
}

void compareFiles::printStatisticsFileAcl(int numAminos, float *compareVect) {

  float refer, *vectAux = new float[numAminos];
  int i, num;

  utils::copyVect(compareVect, vectAux, numAminos);
  utils::quicksort(vectAux, 0, numAminos-1);

  cout << "N.Col\t%Length\t\tNum\tValue" << endl;
  cout.precision(10);

  refer = vectAux[0];
  num = 1;

  for(i = 1; i < numAminos; i++) {

    if(refer != vectAux[i]) {      
      cout << i << "\t" << setw(10) << ((float) i/numAminos * 100.0)
           << "\t" << num  << "\t" << setw(15) << refer << endl;
      refer = vectAux[i]; 
      num = 1;
    }
    else num++;
  }

  cout << i << "\t" << setw(10)  << ((float) i/numAminos * 100.0) 
       << "\t" << num  << "\t" << setw(15) << refer << endl;

  delete [] vectAux;
}


int compareFiles::algorithm(alignment **vectAlignments, char **fileNames, float *columnsValue, int numAlignments) {
	
  int i, j, k, l, m, pairSeq = 0, acert = 0, alig = 0;
  int *numSpeciesAlig, *numAminosAlig;
  int *columnSeqMatrix = NULL, *columnSeqMatrixAux = NULL;
  float max = 0, value = 0, **vectAciertos;
	
  numSpeciesAlig = new int[numAlignments];
  numAminosAlig =  new int[numAlignments];
		
  vectAciertos = new float*[numAlignments];
	
  for(i = 0; i < numAlignments; i++) {
    numSpeciesAlig[i] = vectAlignments[i] -> getNumSpecies(); 
    numAminosAlig[i] =  vectAlignments[i] -> getNumAminos();		
  }
	
  for(i = 0; i < numAlignments; i++, value = 0) {
  	
    cout << endl;
		
    delete[] columnSeqMatrix;	
    columnSeqMatrix = new int[numSpeciesAlig[i]];
    utils::initlVect(columnSeqMatrix, numSpeciesAlig[i], 0);		

    vectAciertos[i] = new float[numAminosAlig[i]];
    utils::initlVect(vectAciertos[i], numAminosAlig[i], 0);

    for(j = 0; j < numAminosAlig[i]; j++, pairSeq = 0, acert = 0) {
      vectAlignments[i] -> getColumnSeqMatrix(j, columnSeqMatrix);	
		
      for(k = 0; k < numSpeciesAlig[i]; k++) {
			
        if(columnSeqMatrix[k] != 0) {
			
          for(l = 0; l < numAlignments; l++) {
            if((l != i) && (k < numSpeciesAlig[l])) {

	      delete[] columnSeqMatrixAux;	
              columnSeqMatrixAux = new int[numSpeciesAlig[l]];
              utils::initlVect(columnSeqMatrixAux, numSpeciesAlig[l], 0);
  	
              vectAlignments[l] -> getColumnSeqMatrix(columnSeqMatrix[k], k, columnSeqMatrixAux);
		
              for(m = k+1; (m < numSpeciesAlig[i]) && (m < numSpeciesAlig[l]); m++)
                if(columnSeqMatrix[m] != 0) {
                  if(columnSeqMatrix[m] == columnSeqMatrixAux[m]) 
                    acert++;
                  pairSeq++;
		}
            }
          }
        } 
      }
      if(pairSeq != 0) {
         vectAciertos[i][j] += ((1.0*acert)/pairSeq);
         value += vectAciertos[i][j];
      }
    }		

    cout << "File:\t" << fileNames[i] << endl << "Values:" << endl << "\tSpecies: " 
         << numSpeciesAlig[i] << "\tAminos: " << numAminosAlig[i] <<  "\tPond. Hits: " << setw(8) 
         << value << "\t%Simil: " << value/numAminosAlig[i] << endl;

    if((value/numAminosAlig[i]) > max) {
      alig = i;
      max = value/numAminosAlig[i];
    } 
  }

  cout << endl << "File Selected: " << fileNames[alig] << " Value: " << max << endl << endl;

  if(columnsValue != NULL)
    for(i = 0; i < numAminosAlig[alig]; i++) 
      columnsValue[i] = vectAciertos[alig][i];

  return alig;
}

int compareFiles::algorithm(alignment **vectAlignments, int numAlignments, float *columnsValue) {
	
  int i, j, k, l, m, acert, pairSeq, alig = 0;
  int *numSpeciesAlig, *numAminosAlig;
  int *columnSeqMatrix = NULL, *columnSeqMatrixAux = NULL;
  float max = 0, value = 0, **vectAciertos;
	
  numSpeciesAlig = new int[numAlignments];
  numAminosAlig =  new int[numAlignments];
		
  vectAciertos = new float*[numAlignments];
	
  for(i = 0; i < numAlignments; i++) {
    numSpeciesAlig[i] = vectAlignments[i] -> getNumSpecies(); 
    numAminosAlig[i] =  vectAlignments[i] -> getNumAminos();		
  }
	
  for(i = 0; i < numAlignments; i++, value = 0) {
  			
    delete[] columnSeqMatrix;	
    columnSeqMatrix = new int[numSpeciesAlig[i]];
    utils::initlVect(columnSeqMatrix, numSpeciesAlig[i], 0);		

    vectAciertos[i] = new float[numAminosAlig[i]];
    utils::initlVect(vectAciertos[i], numAminosAlig[i], 0);

    for(j = 0, pairSeq = 0, acert = 0; j < numAminosAlig[i]; j++, pairSeq = 0, acert = 0) {
      vectAlignments[i] -> getColumnSeqMatrix(j, columnSeqMatrix);	
		
      for(k = 0; k < numSpeciesAlig[i]; k++) {
			
        if(columnSeqMatrix[k] != 0) {
			
          for(l = 0; l < numAlignments; l++) {
            if((l != i) && (k < numSpeciesAlig[l])) {

	      delete[] columnSeqMatrixAux;	
              columnSeqMatrixAux = new int[numSpeciesAlig[l]];
              utils::initlVect(columnSeqMatrixAux, numSpeciesAlig[l], 0);
  	
              vectAlignments[l] -> getColumnSeqMatrix(columnSeqMatrix[k], k, columnSeqMatrixAux);
		
              for(m = k+1; (m < numSpeciesAlig[i]) && (m < numSpeciesAlig[l]); m++)
                if(columnSeqMatrix[m] != 0) {
                  if(columnSeqMatrix[m] == columnSeqMatrixAux[m]) 
                    acert++;
                  pairSeq++;
		}
            }
          }
        } 
      }
      if(pairSeq != 0) {
         vectAciertos[i][j] += ((1.0*acert)/pairSeq);
         value += vectAciertos[i][j];
      }
    }		
    if((value/numAminosAlig[i]) > max) {
      alig = i;
      max = value/numAminosAlig[i];
    } 
  }

  if(columnsValue != NULL)
    for(i = 0; i < numAminosAlig[alig]; i++) 
      columnsValue[i] = vectAciertos[alig][i];

  return alig;
}

